<?php
use CRM_AfformLog_ExtensionUtil as E;

class CRM_AfformLog_BAO_AfformLog extends CRM_AfformLog_DAO_AfformLog {

  /**
   * Create a new AfformLog based on array-data
   *
   * @param array $params key-value pairs
   * @return CRM_AfformLog_DAO_AfformLog|NULL
   */
  public static function create($params) {
    $className = 'CRM_AfformLog_DAO_AfformLog';
    $entityName = 'AfformLog';
    $hook = empty($params['id']) ? 'create' : 'edit';

    foreach (['created_date', 'created_id'] as $fieldName) {
      if (!CRM_Core_Permission::check('administer afform logs') && isset($params[$fieldName])) {
        throw new \API_Exception("Insufficient privileges to modify {$fieldName}");
      }
    }

    switch ($hook) {
      case 'create':
        if (!isset($params['created_id'])) {
          $params['created_id'] = CRM_Core_Session::getLoggedInContactID();
        }
        if (!isset($params['created_date'])) {
          $params['created_date'] = CRM_Utils_Time::getTime('Y-m-d H:i:s');
        }
        break;
    }

    if (is_array($params['data'])) {
      $params['data'] = json_encode($params['data']);
    }

    CRM_Utils_Hook::pre($hook, $entityName, CRM_Utils_Array::value('id', $params), $params);
    $instance = new $className();
    $instance->copyValues($params);
    $instance->save();
    CRM_Utils_Hook::post($hook, $entityName, $instance->id, $instance);

    return $instance;
  }

  /**
   * @return array
   */
  public static function getFormNames() {
    /** @var \CRM_Afform_AfformScanner $scanner */
    $scanner = Civi::service('afform_scanner');
    $keys = array_keys($scanner->findFilePaths());
    return array_combine($keys, $keys);
  }

  public static function getLogTypes() {
    return [
      'read' => E::ts('Read (basic header)'),
      'read_detail' => E::ts('Read (detailed data)'),
      'write_start' => E::ts('Write (start)'),
      'write_error' => E::ts('Write (error)'),
      'write' => E::ts('Write (final)'),
    ];
  }

}
